clear;
clc;
clearglobal;

D=0.0018;
J=0.00045;
k=0.015;
R=1.0;
L=0.003;

//r1 und r2 so whlen, das eine doppelte reelle Polstelle bei -10 entsteht:
doppelpolstelle = -20
r2 = (2*(-doppelpolstelle)-(D/J))*L-R
r1 = J*L*(1/k)*(-(D/J)*(R+r2)/L + 0.25*((R+r2)/L+D/J)^2)-k

//Stationre Werte fr i und omega bestimmen fr u=5Volt:
u = 5;
A = [-D/J,k/J;-k/L,-R/L];
b = [0;u/L]
y_stat = inv(A)*(-b);
w_stat = y_stat(1,1)
i_stat = y_stat(2,1)

//Rckfhrung auf tatschliche Systemmatrix und "Rest" und
//Simulation mit und ohne Regler:

//ungeregelt:
function f = rechteSeite(t,y)
    u=5;
    B=[0;1/L]
    f=A*y+B*u;
endfunction


function f = rechteSeite3(t,y)
    AA = [-D/J,k/J;-k/L,-R/L];
    RR=[r1,r2];
    BB=[0;1/L];
    CC=[0;1];
    pp=(((k+r1)*w_stat+(R+r2)*i_stat)/L);
    f=(A-BB*RR)*y + CC*pp;
endfunction

//ohne Matrix-Schreibweise:
function f = rechteSeite4(t,y)
    u = -r1*(y(1,1)-w_stat)-r2*(y(2,1)-i_stat)+k*w_stat+R*i_stat;
    f(1,1)=(-D/J)*y(1,1)+(k/J)*y(2,1);
    f(1,2)=(-k/L)*y(1,1)+(-R/L)*y(2,1)+(1/L)*u;
endfunction

//Beobachter 0, Ideal:
function f = rechteSeite5(t,y)
//i nicht verfgbar => wird durch Simulation gewonnen, also y4 statt y2:
    u = -r1*(y(1,1)-w_stat)-r2*(y(4,1)-i_stat)+k*w_stat+R*i_stat;

//"Reales" System:
    f(1,1)=(-D/J)*y(1,1)+(k/J)*y(2,1);
    f(1,2)=(-k/L)*y(1,1)+(-R/L)*y(2,1)+(1/L)*u;

//"simuliertes" System (hier identisch, AB bekannt!, das ist unrealistisch):
    f(1,3)=(-D/J)*y(3,1)+(k/J)*y(4,1);
    f(1,4)=(-k/L)*y(3,1)+(-R/L)*y(4,1)+(1/L)*u;
endfunction

//Beobachter 1: Rckfhrung der Zustandsdifferenz y-ysim:
function f = rechteSeite6(t,y)
//i nicht verfgbar => wird durch Simulation gewonnen, also y4 statt y2:
    u = -r1*(y(3,1)-w_stat)-r2*(y(4,1)-i_stat)+k*w_stat+R*i_stat;

//"Reales" System:
    f(1,1)=(-D/J)*y(1,1)+(k/J)*y(2,1);
    f(1,2)=(-k/L)*y(1,1)+(-R/L)*y(2,1)+(1/L)*u;

//"simuliertes" System (hier identisch, AB verkehrt, aber Rckfhrung der Zustandsdifferenz):
    l1=5;
    l2=5;
    f(1,3)=(-D/J)*y(3,1)+(k/J)*y(4,1)            +l1*(y(1,1)-y(3,1));
    f(1,4)=(-k/L)*y(3,1)+(-R/L)*y(4,1)+(1/L)*u   +l2*(y(2,1)-y(4,1));
endfunction


//Beobachter 2: y2==Stromstrke i ist nicht mebar:
function f = rechteSeite7(t,y)
//i nicht verfgbar => wird durch Simulation gewonnen, also y4 statt y2:
    u = -r1*(y(1,1)-w_stat)-r2*(y(4,1)-i_stat)+k*w_stat+R*i_stat;

//"Reales" System:
    f(1,1)=(-D/J)*y(1,1)+(k/J)*y(2,1);
    f(1,2)=(-k/L)*y(1,1)+(-R/L)*y(2,1)+(1/L)*u;

//"simuliertes" System (hier identisch, AB verkehrt, aber Rckfhrung der Zustandsdifferenz):
    l1=5;
    l2=0; //zu Null gesetzt, weil nicht mebar.
    f(1,3)=(-D/J)*y(3,1)+(k/J)*y(4,1)            +l1*(y(1,1)-y(3,1));
    f(1,4)=(-k/L)*y(3,1)+(-R/L)*y(4,1)+(1/L)*u   +l2*(y(2,1)-y(4,1));
endfunction


t = 0:0.01:1;
y0 = [0,0]';
yb0 = [0,0,0,0]';
ybb0 = [0,0,20,20]';
t0   = 0;
y    = ode(y0,t0,t,rechteSeite);
y3   = ode(y0,t0,t,rechteSeite3);
y4   = ode(y0,t0,t,rechteSeite4);
y5   = ode(yb0,t0,t,rechteSeite5);
y5b  = ode(ybb0,t0,t,rechteSeite5); //falsche AB
y6   = ode(ybb0,t0,t,rechteSeite6); //falsche AB, aber Zustandsdifferenz Rckfhrung
y7   = ode(ybb0,t0,t,rechteSeite7); //falsche AB, aber Zustandsdifferenz Rckfhrung, aber y2 nicht mebar.

plot(t,y(1,:)','blu',t,y3(1,:)','gre',t,y5b(1,:)','red',t,y6(1,:)','cya',t,y7(1,:)','mag');
a = gca();
a.x_label.text = 't/s';
a.y_label.text = 'y';
a.title.text = 'Vergleich Sprungantwort (u=5Volt) mit Zustandsregler';
hl=legend(['Sprungantwort u=5Volt';'Zustandsregler, pol=-20';'Beobachter, falsche AB';'Beob. Rueckf. y-ysim';'Beob. i nicht mebar'],a=1);

 


